# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Models for webhook receivers."""

from __future__ import absolute_import

import re

from flask import current_app, request, url_for

from . import signatures
from .proxies import current_webhooks


#
# Errors
#
class WebhookError(Exception):
    """General webhook error."""

    pass


class ReceiverDoesNotExist(WebhookError):
    """Raised when receiver does not exist."""

    pass


class InvalidPayload(WebhookError):
    """Raised when the payload is invalid."""

    pass


class InvalidSignature(WebhookError):
    """Raised when the signature does not match."""

    pass


#
# Models
#
class Receiver(object):
    """Base class for a webhook receiver.

    A receiver is responsible for receiving and extracting a payload from a
    request, and passing it on to a method which can handle the event
    notification.
    """

    def __init__(self, fn, signature=''):
        """Initialize a receiver with a callable and a signature."""
        self._callable = fn
        self.signature = signature

    @classmethod
    def get(cls, receiver_id):
        """Return registered receiver."""
        try:
            return current_webhooks.receivers[receiver_id]
        except KeyError:
            raise ReceiverDoesNotExist(receiver_id)

    @classmethod
    def all(cls):
        """Return all registered receivers."""
        return current_webhooks.receivers

    @classmethod
    def register(cls, receiver_id, receiver):
        """Register a webhook receiver.

        :param receiver_id: Receiver ID used in the URL.
        :param receiver: ``Receiver`` instance.
        """
        current_webhooks.register(receiver_id, receiver)

    @classmethod
    def unregister(cls, receiver_id):
        """Unregister an already registered webhook.

        :param receiver_id: Receiver ID used when registering.
        """
        current_webhooks.unregister(receiver_id)

    @classmethod
    def get_hook_url(cls, receiver_id, access_token):
        """Get URL for webhook.

        In debug and testing mode the hook URL can be overwritten using
        ``WEBHOOKS_DEBUG_RECEIVER_URLS`` configuration variable to allow
        testing webhooks via services such as e.g. Ultrahook.

        .. code-block:: python

            WEBHOOKS_DEBUG_RECEIVER_URLS = dict(
                github='http://github.userid.ultrahook.com',
            )
        """
        cls.get(receiver_id)
        # Allow overwriting hook URL in debug mode.
        if (current_app.debug or current_app.testing) and \
           current_app.config.get('WEBHOOKS_DEBUG_RECEIVER_URLS', None):
            url_pattern = current_app.config[
                'WEBHOOKS_DEBUG_RECEIVER_URLS'].get(receiver_id, None)
            if url_pattern:
                return url_pattern % dict(token=access_token)
        return url_for(
            'invenio_webhooks.event_list',
            receiver_id=receiver_id,
            access_token=access_token,
            _external=True
        )

    #
    # Instance methods (override if needed)
    #
    def consume_event(self, user_id):
        """Consume a webhook event by calling the associated callable."""
        event = self._create_event(user_id)
        self._callable(event)

    def _create_event(self, user_id):
        """Create a new webhook event."""
        return Event(
            user_id,
            payload=self.extract_payload()
        )

    def check_signature(self):
        """Check signature of signed request."""
        if not self.signature:
            return True
        signature_value = request.headers.get(self.signature, None)
        if signature_value:
            validator = 'check_' + re.sub(r'[-]', '_', self.signature).lower()
            check_signature = getattr(signatures, validator)
            if check_signature(signature_value, request.data):
                return True
        return False

    def extract_payload(self):
        """Extract payload from request."""
        if not self.check_signature():
            raise InvalidSignature('Invalid Signature')
        if request.content_type == 'application/json':
            return request.get_json()
        elif request.content_type == 'application/x-www-form-urlencoded':
            return dict(request.form)
        raise InvalidPayload(request.content_type)


class CeleryReceiver(Receiver):
    """Asynchronous receiver.

    Receiver which will fire a celery task to handle payload instead of running
    it synchronously during the request.
    """

    def __init__(self, task_callable, signature='', **options):
        """Initialize a celery receiver."""
        super(CeleryReceiver, self).__init__(task_callable, signature)
        self._task = task_callable
        self._options = options
        from celery import Task
        assert isinstance(self._task, Task)

    def consume_event(self, user_id):
        """Consume a webhook event by firing celery task."""
        event = self._create_event(user_id)
        self._task.apply_async(args=[event.__getstate__()], **self._options)


class Event(object):
    """Incoming webhook event data.

    Represents webhook event data which consists of a payload and a user id.
    """

    def __init__(self, user_id=None, payload=None):
        """Initialize an event with user identifier and payload."""
        self.user_id = user_id
        self.payload = payload

    def __getstate__(self):
        """Pickle current state."""
        return dict(
            user_id=self.user_id,
            payload=self.payload,
        )

    def __setstate__(self, state):
        """Unpickle state."""
        self.user_id = state['user_id']
        self.payload = state['payload']



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Invenio module for processing webhook events."""

from __future__ import absolute_import, print_function

import json
from functools import wraps

from flask import Blueprint, abort, jsonify, request
from flask.views import MethodView
from flask_babelex import lazy_gettext as _
from invenio_oauth2server import require_api_auth, require_oauth_scopes
from invenio_oauth2server.models import Scope

from .models import InvalidPayload, Receiver, ReceiverDoesNotExist, \
    WebhookError

blueprint = Blueprint('invenio_webhooks', __name__)

#
# Required scope
#
webhooks_event = Scope(
    'webhooks:event',
    group='Notifications',
    help_text=_('Allow notifications from external service.'),
    internal=True,
)


#
# Default decorators
#
def error_handler(f):
    """Decorator to handle exceptions."""
    @wraps(f)
    def inner(*args, **kwargs):
        try:
            return f(*args, **kwargs)
        except ReceiverDoesNotExist:
            return jsonify(
                status=404,
                description="Receiver does not exists."
            ), 404
        except InvalidPayload as e:
            return jsonify(
                status=415,
                description="Receiver does not support the"
                            " content-type '%s'." % e.args[0]
            ), 415
        except WebhookError:
            return jsonify(
                status=500,
                description="Internal server error"
            ), 500
    return inner


#
# REST Resources
#
class ReceiverEventListResource(MethodView):
    """Receiver event hook."""

    @require_api_auth()
    @require_oauth_scopes('webhooks:event')
    @error_handler
    def post(self, receiver_id=None):
        """Handle POST request."""
        receiver = Receiver.get(receiver_id)
        receiver.consume_event(request.oauth.access_token.user_id)
        return jsonify(status=202, message='Accepted'), 202

    def options(self, receiver_id=None):
        """Handle OPTIONS request."""
        abort(405)


#
# Register API resources
#
view = ReceiverEventListResource.as_view('event_list')
blueprint.add_url_rule(
    '/receivers/<string:receiver_id>/events/',
    view_func=view,
)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Invenio module for processing webhook events."""

from __future__ import absolute_import, print_function

import pkg_resources

from . import config
from .views import blueprint


class _WebhooksState(object):
    """Webhooks state storing registered receivers."""

    def __init__(self, app, entry_point_group=None):
        """Initialize state."""
        self.app = app
        self.receivers = {}

        if entry_point_group:
            self.load_entry_point_group(entry_point_group)

    def register(self, receiver_id, receiver):
        """Register a receiver."""
        assert receiver_id not in self.receivers
        self.receivers[receiver_id] = receiver

    def unregister(self, receiver_id):
        """Unregister a receiver by its id."""
        del self.receivers[receiver_id]

    def load_entry_point_group(self, entry_point_group):
        """Load actions from an entry point group."""
        for ep in pkg_resources.iter_entry_points(group=entry_point_group):
            self.register(ep.name, ep.load())


class InvenioWebhooks(object):
    """Invenio-Webhooks extension."""

    def __init__(self, app=None, **kwargs):
        """Extension initialization."""
        if app:
            self.init_app(app, **kwargs)

    def init_app(self, app, entry_point_group='invenio_webhooks.receivers'):
        """Flask application initialization."""
        self.init_config(app)
        app.register_blueprint(
            blueprint,
            url_prefix=app.config['WEBHOOKS_URL_PREFIX'],
        )
        state = _WebhooksState(app, entry_point_group=entry_point_group)
        self._state = app.extensions['invenio-webhooks'] = state

    def init_config(self, app):
        """Initialize configuration."""
        app.config.setdefault(
            'WEBHOOKS_BASE_TEMPLATE',
            app.config.get('BASE_TEMPLATE',
                           'invenio_webhooks/base.html'))

        for k in dir(config):
            if k.startswith('WEBHOOKS_'):
                app.config.setdefault(k, getattr(config, k))



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.


"""Pytest configuration."""

from __future__ import absolute_import, print_function

import os

import pytest
from flask import Flask
from flask_babelex import Babel
from flask_breadcrumbs import Breadcrumbs
from flask_celeryext import FlaskCeleryExt
from flask_cli import FlaskCLI
from flask_mail import Mail
from flask_menu import Menu
from invenio_accounts import InvenioAccounts
from invenio_accounts.views import blueprint as accounts_blueprint
from invenio_db import InvenioDB, db
from invenio_oauth2server import InvenioOAuth2Server
from invenio_oauth2server.models import Token
from invenio_oauth2server.views import server_blueprint, settings_blueprint

from invenio_webhooks import InvenioWebhooks
from invenio_webhooks.models import Receiver


@pytest.fixture()
def app(request):
    """Flask application fixture."""
    app = Flask('testapp')
    app.config.update(
        TESTING=True,
        CELERY_ALWAYS_EAGER=True,
        CELERY_CACHE_BACKEND="memory",
        CELERY_EAGER_PROPAGATES_EXCEPTIONS=True,
        CELERY_RESULT_BACKEND="cache",
        LOGIN_DISABLED=False,
        SECRET_KEY='test_key',
        SQLALCHEMY_TRACK_MODIFICATIONS=True,
        SQLALCHEMY_DATABASE_URI=os.getenv('SQLALCHEMY_DATABASE_URI',
                                          'sqlite:///test.db'),
        WTF_CSRF_ENABLED=False,
        OAUTHLIB_INSECURE_TRANSPORT=True,
        OAUTH2_CACHE_TYPE='simple',
        SECURITY_PASSWORD_HASH='plaintext',
        SECURITY_PASSWORD_SCHEMES=['plaintext'],
    )
    FlaskCLI(app)
    FlaskCeleryExt(app)
    Babel(app)
    Mail(app)
    Menu(app)
    Breadcrumbs(app)
    InvenioDB(app)
    InvenioAccounts(app)
    app.register_blueprint(accounts_blueprint)
    InvenioOAuth2Server(app)
    app.register_blueprint(server_blueprint)
    app.register_blueprint(settings_blueprint)
    InvenioWebhooks(app)

    with app.app_context():
        db.create_all()

    def teardown():
        with app.app_context():
            db.drop_all()

    request.addfinalizer(teardown)
    return app


@pytest.fixture
def tester(app):
    """Fixture that contains the test data for models tests."""
    with app.app_context():
        datastore = app.extensions['security'].datastore
        with db.session.begin_nested():
            tester = datastore.create_user(
                email='info@invenio-software.org', password='tester',
            )
    return tester


@pytest.fixture
def access_token(app, tester):
    with app.app_context():
        return Token.create_personal(
            'test-personal-{0}'.format(tester.id),
            tester.id,
            scopes=['webhooks:event'],
            is_internal=True,
        ).access_token


@pytest.fixture
def access_token_no_scope(app, tester):
    with app.app_context():
        return Token.create_personal(
            'test-personal-{0}'.format(tester.id),
            tester.id,
            scopes=[''],
            is_internal=True,
        ).access_token


@pytest.fixture()
def receiver(app):
    """Register test receiver."""
    calls = []

    def consumer(event):
        calls.append(event)

    test_receiver = Receiver(consumer)
    test_receiver.calls = calls

    with app.app_context():
        Receiver.register('test-receiver', test_receiver)

    return test_receiver



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2014 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

from __future__ import absolute_import

import json

from flask import url_for

from invenio_webhooks.models import Receiver


def make_request(access_token, client_func, endpoint, urlargs=None, data=None,
                 is_json=True, code=None, headers=None,
                 follow_redirects=False):
    """Make a request to the API endpoint.

    Ensures request looks like they arrive on CFG_SITE_SECURE_URL.
    That header "Contet-Type: application/json" is added if the parameter
    is_json is True
    :param endpoint: Endpoint passed to url_for.
    :param urlargs: Keyword args passed to url_for
    :param data: Request body, either as a dictionary if ``is_json`` is
        True, or as a string if ``is_json`` is False
    :param headers: List of headers for the request
    :param code: Assert response status code
    :param follow_redirects: Whether to follow redirects.
    """
    urlargs = urlargs or {}
    urlargs['access_token'] = access_token

    if headers is None:
        headers = [('content-type', 'application/json')] if is_json else []

    if data is not None:
        request_args = dict(
            data=json.dumps(data) if is_json else data,
            headers=headers,
        )
    else:
        request_args = {}

    url = url_for(endpoint, **urlargs)
    response = client_func(
        url,
        follow_redirects=follow_redirects,
        **request_args
    )
    if code is not None:
        assert code == response.status_code
    return response


def test_405_methods(app, tester, access_token):
    with app.test_request_context():
        with app.test_client() as client:
            methods = [
                client.get, client.put, client.delete, client.head,
                client.options, client.patch
            ]

            for client_func in methods:
                make_request(
                    access_token,
                    client_func,
                    'invenio_webhooks.event_list',
                    urlargs=dict(receiver_id='test-receiver'),
                    code=405,
                )


def test_webhook_post_unregistered(app, tester, access_token):
    with app.test_request_context():
        with app.test_client() as client:
            make_request(
                access_token,
                client.post,
                'invenio_webhooks.event_list',
                urlargs=dict(receiver_id='test-receiver'),
                code=404,
            )


def test_webhook_post(app, tester, access_token, receiver):
    with app.test_request_context():
        with app.test_client() as client:
            payload = dict(somekey='somevalue')
            make_request(
                access_token,
                client.post,
                'invenio_webhooks.event_list',
                urlargs=dict(receiver_id='test-receiver'),
                data=payload,
                code=202,
            )

            assert 1 == len(receiver.calls)
            assert tester.id == receiver.calls[0].user_id
            assert payload == receiver.calls[0].payload

            # Test invalid payload
            import pickle
            payload = dict(somekey='somevalue')
            make_request(
                access_token,
                client.post,
                'invenio_webhooks.event_list',
                urlargs=dict(receiver_id='test-receiver'),
                data=pickle.dumps(payload),
                is_json=False,
                headers=[('Content-Type', 'application/python-pickle')],
                code=415,
            )

            # Test invalid payload, with wrong content-type
            make_request(
                access_token,
                client.post,
                'invenio_webhooks.event_list',
                urlargs=dict(receiver_id='test-receiver'),
                data=pickle.dumps(payload),
                is_json=False,
                headers=[('Content-Type', 'application/json')],
                code=400,
            )


def test_405_methods_no_scope(app, tester, access_token_no_scope):
    with app.test_request_context():
        with app.test_client() as client:
            methods = [
                client.get, client.put, client.delete, client.head,
                client.options, client.patch
            ]

            for client_func in methods:
                make_request(
                    access_token_no_scope,
                    client_func,
                    'invenio_webhooks.event_list',
                    urlargs=dict(receiver_id='test-receiver'),
                    code=405,
                )


def test_webhook_post_no_scope(app, tester, access_token_no_scope):
    with app.test_request_context():
        r = Receiver(lambda event: event)
        Receiver.register('test-receiver-no-scope', r)

        with app.test_client() as client:
            payload = dict(somekey='somevalue')
            make_request(
                access_token_no_scope,
                client.post,
                'invenio_webhooks.event_list',
                urlargs=dict(receiver_id='test-receiver-no-scope'),
                data=payload,
                code=403,
            )



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Calculate signatures for payloads."""

import hmac
from hashlib import sha1

from flask import current_app


def get_hmac(message):
    """Calculate HMAC value of message using ``WEBHOOKS_SECRET_KEY``.

    :param message: String to calculate HMAC for.
    """
    key = current_app.config['WEBHOOKS_SECRET_KEY']
    hmac_value = hmac.new(
        key.encode('utf-8') if hasattr(key, 'encode') else key,
        message.encode('utf-8') if hasattr(message, 'encode') else message,
        sha1
    ).hexdigest()
    return hmac_value


def check_x_hub_signature(signature, message):
    """Check X-Hub-Signature used by GitHub to sign requests.

    :param signature: HMAC signature extracted from request.
    :param message: Request message.
    """
    hmac_value = get_hmac(message)
    if hmac_value == signature or \
       (signature.find('=') > -1 and
            hmac_value == signature[signature.find('=') + 1:]):
        return True
    return False



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2014 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

from __future__ import absolute_import

import json

import pytest
from celery import shared_task
from flask import url_for

from invenio_webhooks.models import CeleryReceiver, Event, InvalidPayload, \
    InvalidSignature, Receiver, ReceiverDoesNotExist
from invenio_webhooks.signatures import get_hmac


def test_receiver_registration(app):
    calls = []

    def consumer(event):
        calls.append(event)

    @shared_task(ignore_result=True)
    def task_callable(event_state):
        e = Event()
        e.__setstate__(event_state)
        calls.append(e)

    r = Receiver(consumer)
    r_invalid = Receiver(consumer)

    with app.app_context():
        Receiver.register('test-receiver', r)
        Receiver.register('test-invalid', r_invalid)

        assert 'test-receiver' in Receiver.all()
        assert Receiver.get('test-receiver') == r

        # Double registration
        with pytest.raises(AssertionError):
            Receiver.register('test-receiver', r)

        Receiver.unregister('test-receiver')
        assert 'test-receiver' not in Receiver.all()

        Receiver.register('test-receiver', r)

    # JSON payload parsing
    payload = json.dumps(dict(somekey='somevalue'))
    headers = [('Content-Type', 'application/json')]
    with app.test_request_context(headers=headers, data=payload):
        r.consume_event(2)
        assert 1 == len(calls)
        assert json.loads(payload) == calls[0].payload
        assert 2 == calls[0].user_id

        # NOTE legacy test with no clear reason.
        # with pytest.raises(TypeError):
        #     r_invalid.consume_event(2)
        # assert 1 == len(calls)

    # Form encoded values payload parsing
    payload = dict(somekey='somevalue')
    with app.test_request_context(method='POST', data=payload):
        r.consume_event(2)
        assert 2 == len(calls)
        assert dict(somekey=['somevalue']) == calls[1].payload

    # Test invalid post data
    with app.test_request_context(method='POST', data="invaliddata"):
        with pytest.raises(InvalidPayload):
            r.consume_event(2)

    # Test Celery Receiver
    rcelery = CeleryReceiver(task_callable)
    with app.app_context():
        CeleryReceiver.register('celery-receiver', rcelery)

    # Form encoded values payload parsing
    payload = dict(somekey='somevalue')
    with app.test_request_context(method='POST', data=payload):
        rcelery.consume_event(1)
        assert 3 == len(calls)
        assert dict(somekey=['somevalue']) == calls[2].payload
        assert 1 == calls[2].user_id


def test_unknown_receiver(app):
    """Raise when receiver does not exist."""
    with app.app_context():
        with pytest.raises(ReceiverDoesNotExist):
            Receiver.get('unknown')


def test_hookurl(app, receiver):
    with app.test_request_context():
        assert Receiver.get_hook_url('test-receiver', 'token') == url_for(
            'invenio_webhooks.event_list',
            receiver_id='test-receiver',
            access_token='token',
            _external=True
        )

    app.config['WEBHOOKS_DEBUG_RECEIVER_URLS'] = {
        'test-receiver': 'http://test.local/?access_token=%(token)s'
    }

    with app.test_request_context():
        assert 'http://test.local/?access_token=token' == \
            Receiver.get_hook_url('test-receiver', 'token')


def test_signature_checking(app):
    """Check signatures for payload."""
    calls = []

    def consumer(event):
        calls.append(event)

    r = Receiver(consumer, signature='X-Hub-Signature')
    with app.app_context():
        Receiver.register('test-receiver-sign', r)

    # check correct signature
    payload = json.dumps(dict(somekey='somevalue'))
    with app.app_context():
        headers = [('Content-Type', 'application/json'),
                   ('X-Hub-Signature', get_hmac(payload))]
    with app.test_request_context(headers=headers, data=payload):
        r.consume_event(2)
        assert json.loads(payload) == calls[0].payload

    # check signature with prefix
    with app.app_context():
        headers = [('Content-Type', 'application/json'),
                   ('X-Hub-Signature', 'sha1=' + get_hmac(payload))]
    with app.test_request_context(headers=headers, data=payload):
        r.consume_event(2)
        assert json.loads(payload) == calls[1].payload

    # check incorrect signature
    with app.app_context():
        headers = [('Content-Type', 'application/json'),
                   ('X-Hub-Signature', get_hmac("somevalue"))]
    with app.test_request_context(headers=headers, data=payload):
        with pytest.raises(InvalidSignature):
            r.consume_event(2)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Webhooks module."""

WEBHOOKS_URL_PREFIX = '/hooks'
"""URL prefix for Webhooks blueprint."""

WEBHOOKS_DEBUG_RECEIVER_URLS = {}
"""Mapping of receiver id to URL pattern.

This allows generating URLs to an intermediate webhook proxy service like
Ultrahook for testing on development machines:

.. code-block:: python

    WEBHOOKS_DEBUG_RECEIVER_URLS = {
        'github': 'https://hook.user.ultrahook.com/?access_token=%%(token)s'
    }
"""

WEBHOOKS_SECRET_KEY = 'secret_key'
